# -*- coding: utf-8 -*-
"""
Command line action to start a new project
"""
import datetime, os
from string import letters, digits

from argh import arg, CommandError

from optimus.logs import init_logging
from optimus.start_project import ProjectStarter

@arg('name', help="Project's name to use, must be a valid python module name")
@arg('-t', '--template', default="basic", help="A python path to a 'project template' module to use instead of the default one 'optimus.defaults.sample', there is also a sample with i18n at 'optimus.defaults.sample_i18n'.")
@arg('--dry-run', default=False, help="Dry run mode will perform all processus but will not create or modify anything")
@arg('-l', '--loglevel', default='info', choices=['debug','info','warning','error','critical'], help="The minimal verbosity level to limit logs output")
@arg('--logfile', default=None, help="A filepath that if setted, will be used to save logs output")
def init(args):
    """
    Create a new project from a project template
    """
    starttime = datetime.datetime.now()
    # Init, load and builds
    root_logger = init_logging(args.loglevel.upper(), logfile=args.logfile)
    
    # Valid that all characters from the name are : "_" character, letters, 
    # identifier ::=  (letter|"_") (letter | digit | "_")*
    # This is not fully safe, user can create a project name using an installed 
    # Python module that will override it and make some troubles in some case
    if args.name:
        if args.name[0] not in letters:
            root_logger.error("Project name must start with a letter")
            return
        for k in args.name[1:]:
            if k not in letters and k not in digits and k != "_":
                root_logger.error("Project name must only contains letters, digits or '_' character")
                return
    
    if args.dry_run:
        root_logger.warning("'Dry run' mode enabled")
    
    # TODO: optionnal command option to specify another path where the project will 
    #       be created
    project_directory = os.path.abspath(os.getcwd())

    loader = ProjectStarter(project_directory, args.name, dry_run=args.dry_run)
    loader.install(args.template)
    
    endtime = datetime.datetime.now()
    root_logger.info('Done in %s', str(endtime-starttime))



# -*- coding: utf-8 -*-
"""
Command line action to manage translation PO files with pybabel
"""
import datetime, os

from argh import arg

from optimus.logs import init_logging
from optimus.i18n import I18NManager

@arg('-i', '--init', default=False, help="Initialize structure, create template catalog (POT) and initialize catalogs (PO)")
@arg('-u', '--update', default=False, help="Extract translations, update the template catalog (POT) and update the catalogs (PO)")
@arg('-c', '--compile', default=False, help="Process to compilation of catalogs")
@arg('-b', '--backup', default=False, help="Make a backup of the locale directory before doing anything")
@arg('-s', '--settings', default='settings', help="Python path to the settings module")
@arg('-l', '--loglevel', default='info', choices=['debug','info','warning','error','critical'], help="The minimal verbosity level to limit logs output")
@arg('--logfile', default=None, help="A filepath that if setted, will be used to save logs output")
def po(args, init=False, update=False, compile=False):
    """
    Manage catalog for all knowed languages
    """
    starttime = datetime.datetime.now()
    # Init, load and builds
    root_logger = init_logging(args.loglevel.upper(), logfile=args.logfile)
    
    # Only load optimus stuff after the settings module name has been retrieved
    os.environ['OPTIMUS_SETTINGS_MODULE'] = args.settings
    from optimus.conf import settings, import_project_module
    from optimus.utils import display_settings
    
    display_settings(settings, ('DEBUG', 'PROJECT_DIR','SOURCES_DIR','TEMPLATES_DIR','LOCALES_DIR'))
    
    i18n = I18NManager(root_logger, settings)
    
    # TODO: * Copy the actual locale directory to a temporary directory
    #       * If process is success, remove previous backup dir if exists then promote the temp backup dir as the current backup
    #       * If process goes wrong, just remove the temp backup dir
    #       -> Always keeping only one backup dir and allways for a successful process
    if args.backup:
        pass
        
    # NOTE: Should we do this automatically to prevent error on missing files
    #       OR should we only do checking before and abort on the first missing file ?
    if args.init or args.update or args.compile:
        i18n.init_locales_dir()
        i18n.extract(force=args.update)
        i18n.init_catalogs()
    
    if args.update:
        i18n.update_catalogs()
    
    if args.compile:
        i18n.compile_catalogs()
    
    endtime = datetime.datetime.now()
    root_logger.info('Done in %s', str(endtime-starttime))



# -*- coding: utf-8 -*-
"""
Command line action to launch a simple HTTP server rooted on the project build directory
"""
import os, time

from watchdog.observers import Observer

from argh import arg
from argh.exceptions import CommandError

from optimus.logs import init_logging

import cherrypy

@arg('hostname', help="Hostname to bind the server to, use PORT or ADDRESS:PORT")
@arg('-s', '--settings', default='settings', help='Python path to the settings module')
@arg('-l', '--loglevel', default='info', choices=['debug','info','warning','error','critical'], help='The minimal verbosity level to limit logs output')
@arg('--logfile', default=None, help='A filepath that if setted, will be used to save logs output')
@arg('--silent', default=False, help="If setted, logs output won't be printed out")
def runserver(args):
    """
    Launch the project watcher to automatically re-build knowed elements on changes
    """
    root_logger = init_logging(args.loglevel.upper(), printout=not(args.silent), logfile=args.logfile)
    
    # Only load optimus stuff after the settings module name has been retrieved
    os.environ['OPTIMUS_SETTINGS_MODULE'] = args.settings
    from optimus.conf import settings, import_project_module
    from optimus.utils import display_settings
    
    display_settings(settings, ('DEBUG', 'PROJECT_DIR','PUBLISH_DIR','STATIC_DIR','STATIC_URL'))
    
    # Parse given hostname
    address, port = ("127.0.0.1", "80")
    _splits = args.hostname.split(':')
    if len(_splits)>2:
        raise CommandError("Error: Invalid hostname format, too many ':'")
    elif len(_splits)==2:
        address, port = _splits
        if not port or not address:
            raise CommandError("Error: Invalid hostname format, address or port is empty")
    else:
        port = _splits[0]
    
    try:
        int(port)
    except ValueError:
        raise CommandError("Error: Invalid port given: {0}".format(port))
    
    if not os.path.exists(settings.PUBLISH_DIR):
        raise CommandError("Error: Publish directory does not exist yet, you should build it before")
    
    # Run server with publish directory served with tools.staticdir
    print "Running http server on address {address} with port {port}".format(address=address, port=port)
    
    cherrypy.config.update({
        'server.socket_host': address,
        'server.socket_port': int(port),
        'engine.autoreload_on': False,
    })
    
    conf = {
        '/': {
            'tools.staticdir.index': 'index.html',
            'tools.staticdir.on': True,
            'tools.staticdir.dir': settings.PUBLISH_DIR,
        },
    }
    cherrypy.quickstart(None, '/', config=conf)


# -*- coding: utf-8 -*-
"""
Command line action to build a project
"""
import datetime, os

from argh import arg

from optimus.logs import init_logging

@arg('-s', '--settings', default='settings', help="Python path to the settings module")
@arg('-l', '--loglevel', default='info', choices=['debug','info','warning','error','critical'], help="The minimal verbosity level to limit logs output")
@arg('--logfile', default=None, help="A filepath that if setted, will be used to save logs output")
#@arg('--dry-run', default=False, help="Parse page templates, scan them to search their dependancies but don't build them")
def build(args):
    """
    Build elements for a project
    """
    starttime = datetime.datetime.now()
    # Init, load and builds
    root_logger = init_logging(args.loglevel.upper(), logfile=args.logfile)
    
    # Only load optimus stuff after the settings module name has been retrieved
    os.environ['OPTIMUS_SETTINGS_MODULE'] = args.settings
    from optimus.conf import settings, import_project_module
    from optimus.builder.assets import register_assets
    from optimus.builder.pages import PageBuilder
    from optimus.utils import initialize, display_settings
    
    display_settings(settings, ('DEBUG', 'PROJECT_DIR','SOURCES_DIR','TEMPLATES_DIR','PUBLISH_DIR','STATIC_DIR','STATIC_URL'))
    
    if hasattr(settings, 'PAGES_MAP'):
        root_logger.info('Loading external pages map')
        pages_map = import_project_module(settings.PAGES_MAP)
        setattr(settings, 'PAGES', pages_map.PAGES)

    initialize(settings)
    # Assets
    assets_env = register_assets()
    # Pages
    pages_env = PageBuilder(assets_env=assets_env)
    pages_env.build_bulk(settings.PAGES)
    
    endtime = datetime.datetime.now()
    root_logger.info('Done in %s', str(endtime-starttime))



# -*- coding: utf-8 -*-
"""
Settings configuration management
"""
import os, imp, logging, sys

ENVIRONMENT_VARIABLE = "OPTIMUS_SETTINGS_MODULE"

def import_settings(name=None):
    """
    Load the settings, use "os.environ" to find the settings module name if "name" 
    argument is not given
    """
    logger = logging.getLogger('optimus')
    
    if name is None:
        # Stealed from "django.conf"
        try:
            name = os.environ[ENVIRONMENT_VARIABLE]
            if not name: # If it's set but is an empty string.
                raise KeyError
        except KeyError:
            # NOTE: This is arguably an EnvironmentError, but that causes
            # problems with Python's interactive help.
            raise ImportError("Settings cannot be imported, because environment variable %s is undefined." % ENVIRONMENT_VARIABLE)
    
    _settings = import_project_module(name)
    
    # Raise exception if these required settings are not defined
    required_settings = ('DEBUG','SITE_NAME','SITE_DOMAIN','SOURCES_DIR','TEMPLATES_DIR','PUBLISH_DIR','STATIC_DIR','STATIC_URL',)
    missing_settings = []
    for setting_name in required_settings:
        if not hasattr(_settings, setting_name):
            missing_settings.append(setting_name)
    if len(missing_settings)>0:
        logger.error("The following settings are required but not defined in the used settings file: {0}".format(", ".join(missing_settings)))
        raise NameError
    
    # Fill default required settings
    
    # The directory where webassets will store his cache
    if not hasattr(_settings, "WEBASSETS_CACHE"):
        setattr(_settings, "WEBASSETS_CACHE", os.path.join(_settings.PROJECT_DIR, '.webassets-cache'))
    
    # Bundles
    if hasattr(_settings, "EXTRA_BUNDLES"):
        setattr(_settings, "BUNDLES", _settings.EXTRA_BUNDLES)
        logger.warning("'EXTRA_BUNDLES' setting is deprecated and will be removed in futur release, rename it to 'BUNDLES' in your settings.")
    if not hasattr(_settings, "BUNDLES"):
        setattr(_settings, "BUNDLES", {})
    if not hasattr(_settings, "ENABLED_BUNDLES"):
        setattr(_settings, "ENABLED_BUNDLES", _settings.BUNDLES.keys())
    
    # ReSTructuredText parser settings to use when building a RST document
    if not hasattr(_settings, "RST_PARSER_SETTINGS"):
        setattr(_settings, "RST_PARSER_SETTINGS", {
            'initial_header_level': 3,
            'file_insertion_enabled': True,
            'raw_enabled': False,
            'footnote_references': 'superscript',
            'doctitle_xform': False,
        })
    
    # Default directory for translation catalog
    if not hasattr(_settings, "LOCALES_DIR"):
        setattr(_settings, "LOCALES_DIR", os.path.join(_settings.PROJECT_DIR, 'locale'))
    # Default used language
    if not hasattr(_settings, "LANGUAGE_CODE"):
        setattr(_settings, "LANGUAGE_CODE", "en_US")
    # Default available languages to manage
    if not hasattr(_settings, "LANGUAGES"):
        setattr(_settings, "LANGUAGES", (_settings.LANGUAGE_CODE,))
    # Default map for translaction extract with babel
    if not hasattr(_settings, "I18N_EXTRACT_MAP"):
        setattr(_settings, "I18N_EXTRACT_MAP", (
            ('pages.py', 'python'),
            ('*settings.py', 'python'),
            ('**/templates/**.html', 'jinja2'),
        ))
    if not hasattr(_settings, "I18N_EXTRACT_OPTIONS"):
        setattr(_settings, "I18N_EXTRACT_OPTIONS", {
            '**/templates/**.html': {
                'extensions': 'webassets.ext.jinja2.AssetsExtension',
                'encoding': 'utf-8'
            },
        })
    if not hasattr(_settings, "I18N_EXTRACT_SOURCES"):
        setattr(_settings, "I18N_EXTRACT_SOURCES", (_settings.PROJECT_DIR,))
    
    # Python paths for each extensions to use with Jinja2
    if not hasattr(_settings, "JINJA_EXTENSIONS"):
        setattr(_settings, "JINJA_EXTENSIONS", (
            'jinja2.ext.i18n',
        ))
    
    # Python path to the file that contains pages map, this is relative to your project
    if not hasattr(_settings, "PAGES_MAP"):
        setattr(_settings, "PAGES_MAP", "pages")
    
    # Sources files or directory to synchronize within the static directory
    if not hasattr(_settings, "FILES_TO_SYNC"):
        setattr(_settings, "FILES_TO_SYNC", ())
    
    # These are the default watcher settings, you can customize them if you want, uncomment 
    # parts you want to change, usually you'll change only the "pattern" values
    # You don't need to uncomment this if you want to use the watcher with these default 
    # parameters

    # Templates watcher settings
    if not hasattr(_settings, "WATCHER_TEMPLATES_PATTERNS"):
        setattr(_settings, "WATCHER_TEMPLATES_PATTERNS", {})
    # Assets watcher settings
    if not hasattr(_settings, "WATCHER_ASSETS_PATTERNS"):
        setattr(_settings, "WATCHER_ASSETS_PATTERNS", {})

    return _settings
    

def import_project_module(name):
    """
    Load the given module name, only from the current directory (where the CLI has been 
    launched)
    """
    project_directory = os.getcwd()
    
    logger = logging.getLogger('optimus')
    logger.info('Loading "%s" module', name)
    logger.info('Module searched in: %s', project_directory)
    
    # Add the project to the sys.path
    project_name = os.path.basename( os.path.abspath( project_directory ) )
    sys.path.append( os.path.normpath( os.path.join(project_directory, '..') ) )
    # Sys.path is ok, we can import the project
    try:
        project_module = __import__(project_name, '', '', [''])
    except ImportError:
        logger.critical("Unable to load the project named '{0}'".format(project_name))
        raise
    # Cleanup the sys.path of the project path
    sys.path.pop()
    
    fp, pathname, description = imp.find_module(name, [project_directory])
    try:
        settings = imp.load_module(name, fp, pathname, description)
    except:
        logger.critical('Unable to load settings file')
        raise
    finally:
        # Close fp explicitly.
        if fp:
            fp.close()

    return settings

settings = import_settings()


# -*- coding: utf-8 -*-
"""
Command line action to launch the project watcher

TODO: Watcher could bug in case of an unbuilded project in some 
      cases (like in DEBUG=False), this has to be verified
"""
from pathtools.patterns import match_path

from watchdog.observers import Observer
from watchdog.events import PatternMatchingEventHandler

class TemplatesWatchEventHandler(PatternMatchingEventHandler):
    """
    Template changes handler
    """
    def __init__(self, settings, logger, assets_env, pages_env, *args, **kwargs):
        self.settings = settings
        self.logger = logger
        self.assets_env = assets_env
        self.pages_env = pages_env
        
        super(TemplatesWatchEventHandler, self).__init__(*args, **kwargs)
    
    def on_moved(self, event):
        # We are only interested for the destination
        if match_path(event.dest_path, 
                included_patterns=self.patterns,
                excluded_patterns=self.ignore_patterns,
                case_sensitive=self.case_sensitive):
            self.logger.debug("Change detected from a move on: %s", event.dest_path)
            self.build_for_item(event.dest_path)
    
    def on_created(self, event):
        self.logger.debug("Change detected from a create on: %s", event.src_path)
        self.build_for_item(event.src_path)

    def on_modified(self, event):
        self.logger.debug("Change detected from an edit on: %s", event.src_path)
        self.build_for_item(event.src_path)

    def get_relative_path(self, path):
        if path.startswith(self.settings.TEMPLATES_DIR):
            return path[len(self.settings.TEMPLATES_DIR)+1:]
        elif path.startswith(self.settings.SOURCES_DIR):
            return path[len(self.settings.SOURCES_DIR)+1:]
        return path

    def build_for_item(self, path):
        """
        (Re)build all pages using the changed template
        
        ``path`` argument is a template path
        """
        rel_path = self.get_relative_path(path)
        
        # Search in the registry if the file is a knowed template dependancy
        if rel_path in self.pages_env.registry.elements:
            self.logger.debug("Build required for: %s", rel_path)
            
            requires = self.pages_env.registry.get_pages_from_dependency(rel_path)
            self.logger.debug("Requires for rebuild: %s", requires)
            self.pages_env.build_bulk(requires)

class AssetsWatchEventHandler(PatternMatchingEventHandler):
    """
    Assets changes handler
    """
    def __init__(self, settings, logger, assets_env, pages_env, *args, **kwargs):
        self.settings = settings
        self.logger = logger
        self.assets_env = assets_env
        self.pages_env = pages_env
        
        super(AssetsWatchEventHandler, self).__init__(*args, **kwargs)
    
    def on_moved(self, event):
        # We are only interested for the destination
        if match_path(event.dest_path, 
                included_patterns=self.patterns,
                excluded_patterns=self.ignore_patterns,
                case_sensitive=self.case_sensitive):
            self.logger.debug("Change detected from a move on: %s", event.dest_path)
            self.build_for_item(event.dest_path)
    
    def on_created(self, event):
        self.logger.debug("Change detected from a create on: %s", event.src_path)
        self.build_for_item(event.src_path)

    def on_modified(self, event):
        self.logger.debug("Change detected from an edit on: %s", event.src_path)
        self.build_for_item(event.src_path)

    def get_relative_path(self, path):
        if path.startswith(self.settings.TEMPLATES_DIR):
            return path[len(self.settings.TEMPLATES_DIR)+1:]
        elif path.startswith(self.settings.SOURCES_DIR):
            return path[len(self.settings.SOURCES_DIR)+1:]
        return path

    def build_for_item(self, path):
        """
        (Re)build the bundle containing the given asset path, then rebuild all pages 
        because in debug mode webassets use an url from his internal cache that change 
        at each rebuild
        
        ``path`` argument is an asset path
        """
        rel_path = self.get_relative_path(path)
        self.logger.debug("Changed relative file : %s", rel_path)
        
        # Search in the registry if the file is a knowed asset from a bundle
        if rel_path in self.assets_env.optimus_registry.map_dest_to_bundle:
            bundle_name = self.assets_env.optimus_registry.map_dest_to_bundle[rel_path]
            self.logger.debug("Build required for bundle: %s", bundle_name)
            
            urls = self.assets_env[bundle_name].urls()
            self.logger.debug("Rebuilded urls: %s", urls)
            
            # Launch all pages rebuild
            self.pages_env.build_bulk(self.settings.PAGES)



# -*- coding: utf-8 -*-
"""
New project starter
"""
import logging, os, shutil
from string import Template

from optimus.utils import recursive_directories_create, synchronize_assets_sources
from optimus.importlib import import_module
from optimus.samples import TEMPLATE_ALIAS

class ProjectStarter(object):
    """
    Object to create a new project with his settings, directory structure, script, etc..
    
    * root_path: path to the directory where to create the new project
    * name: name of the new project, will be also the dir name of the created project, 
            the must be a valid module name (without spaces, special chars, etc..)
    * dry_run: Dry run mode to perform all tasks but never create anything;
    """
    def __init__(self, root_path, name, dry_run=False):
        self.root_path = root_path
        self.name = name
        self.dry_run = dry_run
        self.logger = logging.getLogger('optimus')
    
    def install(self, projecttemplate_modulepath):
        """
        Install the new project structure and content defined by the specified "project template"
        
        * projecttemplate_modulepath: a python path (aka ``foo.bar``) to the module containing 
          all the "project template" stuff.
        """
        
        project_dir = os.path.join(self.root_path, self.name)
        if os.path.exists(project_dir):
            self.logger.error("Project path allready exists : %s", project_dir)
            return
        
        if projecttemplate_modulepath in TEMPLATE_ALIAS:
            self.logger.debug("Resolved project template alias : %s", projecttemplate_modulepath)
            projecttemplate_modulepath = TEMPLATE_ALIAS[projecttemplate_modulepath]
            
        self.logger.info("Loading the project template from : %s", projecttemplate_modulepath)
        try:
            self.projecttemplate = import_module(projecttemplate_modulepath)
        except ImportError:
            self.logger.error("There is no project template module named '%s'", projecttemplate_modulepath)
            return False
        projecttemplate_path = os.path.abspath(os.path.dirname(self.projecttemplate.__file__))
        
        self.logger.info("Creating new Optimus project '%s' in : %s", self.name, self.root_path)
        if not self.dry_run:
            os.makedirs(project_dir)
        
        self.logger.info("Installing directories structure on : %s", project_dir)
        recursive_directories_create(project_dir, self.projecttemplate.DIRECTORY_STRUCTURE, dry_run=self.dry_run)
        
        self.logger.info("Synchronizing sources on : %s", project_dir)
        for item in self.projecttemplate.FILES_TO_SYNC:
            synchronize_assets_sources(os.path.join(projecttemplate_path, self.projecttemplate.SOURCES_FROM), os.path.join(project_dir, self.projecttemplate.SOURCES_TO), *item, dry_run=self.dry_run)
        
        if hasattr(self.projecttemplate, "LOCALE_DIR"):
            locale_src = os.path.join(projecttemplate_path, self.projecttemplate.LOCALE_DIR)
            locale_dst = os.path.join(project_dir, self.projecttemplate.LOCALE_DIR)
            self.logger.info("Installing messages catalogs")
            if not os.path.exists(locale_src):
                logger.error('Message catalog directory does not exists: %s', locale_src)
            if not self.dry_run:
                shutil.copytree(locale_src, locale_dst)
        
        self.logger.info("Installing default project's files")
        context = {
            'PROJECT_NAME': self.name,
            'SOURCES_FROM': self.projecttemplate.SOURCES_FROM,
        }
        self.install_scripts(project_dir, context)
        
        return True
    
    def install_scripts(self, project_dir, context):
        """
        Write the provided scripts by the "project template"
        """
        projecttemplate_path = os.path.abspath(os.path.dirname(self.projecttemplate.__file__))
        self.logger.debug("Getting files from '%s'", projecttemplate_path)
        
        for item in self.projecttemplate.SCRIPT_FILES:
            template_filepath = os.path.join(projecttemplate_path, item[0])
            destination = os.path.join(project_dir, item[1])
            self.logger.info("* Installing '%s' to '%s'", template_filepath, destination)
            self.write_template_script(template_filepath, destination, context=context)
    
    def write_template_script(self, template_filepath, destination, context={}):
        """
        Write a script from the "project template" to the new project
        """
        # reading template file
        template_fileobject = open(template_filepath, 'r')
        content = Template(template_fileobject.read())
        template_fileobject.close()
        # render content
        content = content.substitute(**context)
        self.logger.debug("  Writing")
        
        if not self.dry_run:
            # check destination path and creating it if needed
            dest_path = os.path.dirname(destination)
            if not os.path.exists(dest_path):
                os.makedirs(dest_path)
            # writing file
            defaultfileobject = open(destination, 'w')
            defaultfileobject.write(content)
            defaultfileobject.close()
        


# -*- coding: utf-8 -*-
"""
I18n management support within Optimus environnment

Assume we only manage "messages.*" files for POT and PO files and no other catalog type.
"""
import datetime, os, tempfile

from babel import Locale
from babel.util import LOCALTZ
from babel.messages.extract import extract_from_dir
from babel.messages.catalog import Catalog
from babel.messages.pofile import read_po, write_po
from babel.messages.mofile import write_mo

class I18NManager(object):
    """
    I18n manager for translation catalogs
    
    Maked to work simply within Optimus environnment, so not all of babel 
    options are used. This way the manager can work cleanly and is more easy 
    to use.
    """
    catalog_name = "messages.{0}"
    catalog_path = "{0}/LC_MESSAGES"
    header_comment = "# Translations template for PROJECT project\n# Created by Optimus"

    def __init__(self, logger, settings):
        self.logger = logger
        self.settings = settings
        self._catalog_template = None

    def get_template_path(self):
        """Return the full path to the catalog template file"""
        return os.path.join(self.settings.LOCALES_DIR, self.catalog_name.format("pot"))

    def get_catalog_dir(self, locale):
        """Return the full path to a translations catalog directory"""
        return os.path.join(self.settings.LOCALES_DIR, self.catalog_path.format(locale))

    def get_catalog_path(self, locale):
        """Return the full path to a translations catalog file"""
        return os.path.join(self.get_catalog_dir(locale), self.catalog_name.format("po"))

    def get_compiled_catalog_path(self, locale):
        """Return the full path to a compiled translations catalog file"""
        return os.path.join(self.get_catalog_dir(locale), self.catalog_name.format("mo"))

    def check_locales_dir(self):
        """Check if LOCALES_DIR directory exists"""
        return os.path.exists(self.settings.LOCALES_DIR)

    def check_template_path(self): 
        """Check if the catalog template exists"""
        return os.path.exists(self.get_template_path())
    
    def check_catalog_path(self, locale):
        """Check if a translations catalog exists"""
        return os.path.exists(self.get_catalog_path(locale))

    def parse_languages(self, languages):
        """
        Allways return a list of locale name from languages even if items are simple 
        string or tuples. If tuple, assume its first item is the locale name to use.
        """
        _f = lambda x: x[0] if isinstance(x, list) or isinstance(x, tuple) else x
        return map(_f, languages)

    def init_locales_dir(self):
        """Create LOCALES_DIR directory if not allready exists"""
        if not self.check_locales_dir():
            self.logger.warning('Locales directory does not exists, creating it')
            os.makedirs(self.settings.LOCALES_DIR)

    @property
    def catalog_template(self):
        """
        Return the catalog template
        
        Get it in memory if allready opened, else if exists open it, else 
        extract it and create it.
        """
        if self._catalog_template is not None:
            return self._catalog_template
        if self.check_template_path():
            fp = open(self.get_template_path(), "r")
            self._catalog_template = read_po(fp)
            fp.close()
            return self._catalog_template
        return self.extract()

    @catalog_template.setter
    def catalog_template(self, value):
        self._catalog_template = value

    @catalog_template.deleter
    def catalog_template(self):
        del self._catalog_template
            
    def safe_write_po(self, catalog, filepath, **kwargs):
        """
        Safely write a PO file
        
        This means that the PO file is firstly created in a temporary file, so 
        if it fails it does not overwrite the previous one, if success the 
        temporary file is moved over the previous one.
        
        Some part of code have been stealed from babel.messages.frontend
        """
        tmpname = os.path.join(os.path.dirname(filepath), tempfile.gettempprefix() + os.path.basename(filepath))
        tmpfile = open(tmpname, 'w')
        try:
            try:
                write_po(tmpfile, catalog, **kwargs)
            finally:
                tmpfile.close()
        except:
            os.remove(tmpname)
            raise

        try:
            os.rename(tmpname, filepath)
        except OSError:
            # We're probably on Windows, which doesn't support atomic
            # renames, at least not through Python
            # If the error is in fact due to a permissions problem, that
            # same error is going to be raised from one of the following
            # operations
            os.remove(filepath)
            shutil.copy(tmpname, filepath)
            os.remove(tmpname)

    def clone_template(self):
        """
        Open the template catalog again to clone it and to be able to modify it without 
        change on the "_catalog_template"
        
        NOTE: does it invalidate get_catalog_template method and the _catalog_template 
        cache usage if after all it is not really usable
        NOTE: seems in fact that processes does not really need to access to a verbatim 
        catalog template, so finally we could do cloning in catalog_template, then be 
        able modify it in extract without any loss for further process
        """
        self.logger.debug('Opening template catalog (POT)')
        fp = open(self.get_template_path(), "r")
        catalog = read_po(fp)
        fp.close()
        return catalog

    def extract(self, force=False):
        """
        Extract translation strings from sources directory with extract rules then 
        create the template catalog with finded translation strings
        
        Only proceed if the template catalog does not exists yet or if 
        ``force`` argument is ``True`` (this will overwrite previous existing 
        POT file)
        
        TODO: actually from the CLI usage this only update POT file when he does not 
        exist, else it keeps untouched, even if there changes or adds in translations
        """
        if force or not self.check_template_path():
            self.logger.info('Proceeding to extraction to update the template catalog (POT)')
            self._catalog_template = Catalog(project=self.settings.SITE_NAME, header_comment=self.header_comment)
            # Follow all paths to search for pattern to extract
            for extract_path in self.settings.I18N_EXTRACT_SOURCES:
                self.logger.debug('Searching for pattern to extract in : {0}'.format(extract_path))
                extracted = extract_from_dir(dirname=extract_path, method_map=self.settings.I18N_EXTRACT_MAP, options_map=self.settings.I18N_EXTRACT_OPTIONS)
                # Proceed to extract from given path
                for filename, lineno, message, comments, context in extracted:
                    filepath = os.path.normpath(os.path.join(os.path.basename(self.settings.SOURCES_DIR), filename))
                    self._catalog_template.add(message, None, [(filepath, lineno)], auto_comments=comments, context=context)
            
            outfile = open(self.get_template_path(), 'wb')
            write_po(outfile, self._catalog_template)
            outfile.close()
            
        return self._catalog_template

    def init_catalogs(self, languages=None):
        """
        For each knowed language, check if his PO file exists, else create it from the POT
        """
        catalog_template = self.clone_template()
        languages = self.parse_languages(languages or self.settings.LANGUAGES)
        for locale in languages:
            translation_dir = self.get_catalog_dir(locale)
            catalog_path = self.get_catalog_path(locale)
            if not self.check_catalog_path(locale):
                self.logger.debug('Init catalog (PO) for language {0} at {1}'.format(locale, catalog_path))
                # write_po from the catalog template
                catalog_template.locale = Locale.parse(locale)
                catalog_template.revision_date = datetime.datetime.now(LOCALTZ)
                catalog_template.fuzzy = False
                
                if not os.path.exists(translation_dir):
                    os.makedirs(translation_dir)
                    
                outfile = open(catalog_path, 'wb')
                write_po(outfile, catalog_template)
                outfile.close()

    def update_catalogs(self, languages=None):
        """
        Update all knowed catalogs from the catalog template
        """
        languages = self.parse_languages(languages or self.settings.LANGUAGES)
        for locale in languages:
            catalog_path = self.get_catalog_path(locale)
            self.logger.info('Updating catalog (PO) for language {0} at {1}'.format(locale, catalog_path))
            # Open the PO file
            infile = open(catalog_path, 'U')
            try:
                catalog = read_po(infile, locale=locale)
            finally:
                infile.close()
            # Update it from the template
            catalog.update(self.catalog_template)
            self.safe_write_po(catalog, catalog_path)

    def compile_catalogs(self, languages=None):
        """
        Compile all knowed catalogs
        """
        languages = self.parse_languages(languages or self.settings.LANGUAGES)
        for locale in languages:
            catalog_path = self.get_catalog_path(locale)
            self.logger.info('Compiling catalog (PO) for language {0} at {1}'.format(locale, catalog_path))
            infile = open(catalog_path, 'r')
            try:
                catalog = read_po(infile, locale)
            finally:
                infile.close()
            
            # Check errors in catalog
            errs = False
            for message, errors in catalog.check():
                for error in errors:
                    errs = True
                    self.logger.warning('Error at line {0}: {1}'.format(message.lineno, error))
            # Don't overwrite the previous MO file if there have been error
            if errs:
                self.logger.error('There has been errors within the catalog, compilation has been aborted')
                break

            outfile = open(self.get_compiled_catalog_path(locale), 'wb')
            try:
                write_mo(outfile, catalog, use_fuzzy=False)
            finally:
                outfile.close()



# -*- coding: utf-8 -*-
"""
Command line action to launch the project watcher
"""
import os, time

from watchdog.observers import Observer

from argh import arg

from optimus.logs import init_logging

@arg('-s', '--settings', default='settings', help='Python path to the settings module')
@arg('-l', '--loglevel', default='info', choices=['debug','info','warning','error','critical'], help='The minimal verbosity level to limit logs output')
@arg('--logfile', default=None, help='A filepath that if setted, will be used to save logs output')
@arg('--silent', default=False, help="If setted, logs output won't be printed out")
def watch(args):
    """
    Launch the project watcher to automatically re-build knowed elements on changes
    """
    root_logger = init_logging(args.loglevel.upper(), printout=not(args.silent), logfile=args.logfile)
    
    # Only load optimus stuff after the settings module name has been retrieved
    os.environ['OPTIMUS_SETTINGS_MODULE'] = args.settings
    from optimus.conf import settings, import_project_module
    from optimus.watchers import TemplatesWatchEventHandler, AssetsWatchEventHandler
    from optimus.builder.assets import register_assets
    from optimus.builder.pages import PageBuilder
    from optimus.utils import initialize, display_settings
    
    display_settings(settings, ('DEBUG', 'PROJECT_DIR','SOURCES_DIR','TEMPLATES_DIR','PUBLISH_DIR','STATIC_DIR','STATIC_URL'))
    
    if hasattr(settings, 'PAGES_MAP'):
        root_logger.info('Loading external pages map')
        pages_map = import_project_module(settings.PAGES_MAP)
        setattr(settings, 'PAGES', pages_map.PAGES)
    
    # Init environments
    assets_env = register_assets()
    pages_env = PageBuilder(assets_env=assets_env)
    
    # add a first build to avoid error on unbuilded project
    if not os.path.exists(settings.PUBLISH_DIR):
        root_logger.info('Seems you never do a first build so do it now')
        pages_env.build_bulk(settings.PAGES)
    
    pages_env.scan_bulk(settings.PAGES)
    
    observer = Observer()
    
    # Templates watcher settings
    watcher_templates_patterns = {
        'patterns': ['*.html'],
        'ignore_patterns': None,
        'ignore_directories': False,
        'case_sensitive': False,
    }
    watcher_templates_patterns.update(settings.WATCHER_TEMPLATES_PATTERNS)
    # Assets watcher settings
    watcher_assets_patterns = {
        'patterns': ['*.css', '*.js', '*.json'],
        'ignore_patterns': None,
        'ignore_directories': False,
        'case_sensitive': False,
    }
    watcher_assets_patterns.update(settings.WATCHER_ASSETS_PATTERNS)
    
    # Init templates and assets event watchers
    templates_event_handler = TemplatesWatchEventHandler(settings, root_logger, assets_env, pages_env, **watcher_templates_patterns)
    if assets_env is not None:
        assets_event_handler = AssetsWatchEventHandler(settings, root_logger, assets_env, pages_env, **watcher_assets_patterns)
    # Registering event watchers and start to watch
    observer.schedule(templates_event_handler, settings.TEMPLATES_DIR, recursive=True)
    if assets_env is not None:
        observer.schedule(assets_event_handler, settings.SOURCES_DIR, recursive=True)
        
    root_logger.warning('Launching the watcher, use CTRL+C to stop it')
    observer.start()
    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()



# -*- coding: utf-8 -*-
"""
Various helpers
"""
import logging, os, shutil

def init_directory(directory):
    """
    Create a new directory if it does not allready exists
    """
    logger = logging.getLogger('optimus')
    if not os.path.exists(directory):
        logger.debug('Creating directory: %s', directory)
        os.makedirs(directory)
        return True
    return False

def recursive_directories_create(project_directory, structure, dry_run=False):
    """
    Recursive directory create from a "tree list"
    
    Sample tree list : ::
    
        structure = [
            [
                'sources',
                [
                    ['js'],
                    ['css'],
                ]
            ]
        ]
    """
    logger = logging.getLogger('optimus')
    
    for item in structure:
        if len(item)>0:
            new_dir = item[0]
            path_dir = os.path.join(project_directory, new_dir)
            if not os.path.exists(path_dir):
                logger.info('* Creating new directory : %s', path_dir)
                if not dry_run:
                    os.makedirs(path_dir)
            else:
                logger.warning('* Following path allready exist : %s', path_dir)
        # Follow children directories to create them
        if len(item)>1:
            recursive_directories_create(path_dir, item[1], dry_run=dry_run)
        
    return

def synchronize_assets_sources(from_path, to_path, src, dest, dry_run=False):
    """
    For now, this is just a rmtree/copytree of the given path
    
    TODO: In future, this should be a clean synchronize, like with rsync
          or with making an internal registry of asset files that will be used 
          to make a diff of changes then apply it ?
    
    * ``src`` arg is allways a file path assumed to be located in the 
       path specified with the ``from_path`` argument;
    * ``dst`` is a file path that will be created in the 
       path specified with the ``to_path`` argument;
    * ``src`` arg is allways a file path assumed to be located in the 
    * ``dst`` is a file path that will be in 
    ``settings.``.
    """
    logger = logging.getLogger('optimus')
    source = os.path.join(from_path, src)
    if not os.path.exists(source):
        logger.warning('The given source does not exist and so can not be synchronized : %s', source)
        return
    
    destination = os.path.join(to_path, src)
    if os.path.exists(destination):
        logger.debug('Removing old asset destination: %s', destination)
        if not dry_run:
            shutil.rmtree(destination)
    logger.debug('Synchronizing asset from "%s" to "%s"', source, destination)
    if not dry_run:
        shutil.copytree(source, destination)

def initialize(settings):
    """
    Init the needed directory structure
    """
    init_directory(settings.STATIC_DIR)
    init_directory(settings.WEBASSETS_CACHE)

    if settings.FILES_TO_SYNC is not None:
        for item in settings.FILES_TO_SYNC:
            synchronize_assets_sources(settings.SOURCES_DIR, settings.STATIC_DIR, *item)

def display_settings(settings, names):
    """
    Helper to display some settings if they are setted
    """
    logger = logging.getLogger('optimus')
    for item in names:
        logger.debug(" - Settings.%s = %s", item, getattr(settings, item, 'NOT SET'))



# -*- coding: utf-8 -*-
"""
Language object used to encapsulate some details on a language

Usage : ::

    class LangFr(LangBase):
        code = 'fr'
        label = 'France'

Or : ::

    lang = LangBase(lang="zh_CN", label="Chinese")
"""
class LangBase(object):
    """
    Language base object to encapsulate the language label, code and other details
    
    Alternative and External code are not really used internally in optimus, there 
    are only for some template usage.
    
    The instance will also supply a "language_name" and "region_name" class attributes, 
    which are the result of splitting the code on two parts. "region_name" is ``None`` 
    by default, as the region name is optional in language identifier.
    
    See http://www.i18nguy.com/unicode/language-identifiers.html for more details on 
    language identifiers.
    """
    label = None # Label to display
    code = None # Internal code
    alt_code = None # Alternative code, will be equal to "code" if not setted
    external_code = None # External code for some external apps, will be equal to "alt_code" if not setted
    
    def __init__(self, code=None, label=None):
        self.code = code or self.code
        
        if self.code is None:
            raise ValueError("Invalid language identifier : You must supply it by the way of 'code' argument or as the 'code' class attribute.")
        if len(self.code.split('-'))>1:
            raise ValueError("Invalid language identifier : Language name and region name must be joined by a '_' not a '-'")
        
        self.language_name, self.region_name = self.split_code(self.code)
        
        self.label = label or self.label or self.code
        
        self.alt_code = self.alt_code or self.code
        self.external_code = self.external_code or self.code
    
    def split_code(self, code):
        items = code.split('_')
        language_name = items[0]
        region_name = None
        if len(items)>1:
            region_name = items[1]
        return language_name, region_name
    
    def __str__(self):
        return self.label.encode('utf-8')
    
    def __unicode__(self):
        return self.label



"""
Default template project to use to create a new project
"""
# Directory structure to create
DIRECTORY_STRUCTURE = [
    # list(dir_name[, children_dir_list])
    [
        'sources',
        [
            ['js'],
            ['css'],
            ['scss'],
            ['images'],
            ['templates'],
        ]
    ]
]

# The directory name that contains 'sources' (assets, templates, images, etc..)
# in the template project
SOURCES_FROM = 'sources'
# The directory name that will contains 'sources' in the new created projects
SOURCES_TO = SOURCES_FROM

# Default sources files or directory to synchronize within the new project 
# sources directory
# The sync task is performed before the writing task
FILES_TO_SYNC = (
    #(SOURCE, DESTINATION)
    ('js', 'js'),
    ('css', 'css'),
    ('scss', 'scss'),
    ('templates', 'templates'),
)

# Script template files
# NOTE: Files are rendered with the ``String.format()`` method, so remember to double 
#       all your '{' and '}', else they will be interpreted as format variable, and 
#       probably raise a Key error
SCRIPT_FILES = [
    ['README.rst', 'README.rst'],
    # Default scripts
    ['scripts/gitignore.tpl', '.gitignore'],
    ['scripts/config.rb.tpl', 'config.rb'],
    ['scripts/__init__.py.tpl', '__init__.py'],
    ['scripts/settings.py.tpl', 'settings.py'],
    ['scripts/prod_settings.py.tpl', 'prod_settings.py'],
    ['scripts/pages.py.tpl', 'pages.py'],
]



from setuptools import setup, find_packages

setup(
    name='Optimus',
    version=__import__('optimus').__version__,
    description=__import__('optimus').__doc__,
    long_description=open('README.rst').read(),
    author='David Thenon',
    author_email='sveetch@gmail.com',
    url='http://pypi.python.org/pypi/Optimus',
    license='MIT',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
        'Topic :: Text Processing :: Markup',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    install_requires=[
        'docutils==0.9.1',
        'Jinja2>=2.6',
        'webassets==0.8',
        'argparse==1.2.1',
        'argh==0.16.0',
        'pathtools==0.1.2',
        'watchdog==0.6.0',
        'rstview==0.1.2',
        'Babel==1.3.0',
        'yuicompressor==2.4.7',
        'cherrypy==3.2.4',
    ],
    #scripts=['bin/optimus'],
    entry_points={
        'console_scripts': [
            'optimus = optimus.cli.console_script:main',
        ]
    },
    include_package_data=True,
    zip_safe=False
)


# -*- coding: utf-8 -*-
"""
Page building with Jinja2
"""
import copy, logging, os

from jinja2 import Environment as Jinja2Environment
from jinja2 import meta as Jinja2Meta
from jinja2 import FileSystemLoader
from webassets.ext.jinja2 import AssetsExtension

import docutils
import docutils.core
import docutils.nodes
import docutils.utils
import docutils.parsers.rst

from rstview import html5writer

# Optional babel import
try:
    from babel import support
except ImportError:
    support = None

from optimus.lang import LangBase
from optimus.conf import settings

class PageViewBase(object):
    """
    Base view object for a page
    
    You can set class attributes at the init if needed
    
    The render method is responsible to rendering the HTML from the template and 
    his context. Actually this is the only used method directly
    
    Only ``lang`` and ``context`` attributes are optional, so take care to set all the 
    required ones because their default value is ``None``. You should not use 
    directly ``PageViewBase``, inherit it in a common object with all attributes setted 
    by default.
    
    Default context will have the following variables :
    
    * page_title: the specified page title 
    * page_destination: the page destination
    * page_lang: the given langage if any
    * page_template_name: the template name used to compile the page HTML
    
    But you can add new variable if needed. The default context variables can not be 
    overriden from the ``context`` class attribute, only from the ``get_context`` class 
    method.
    """
    title = None
    template_name = None
    destination = None
    lang = None
    context = {}
    
    def __init__(self, **kwargs):
        self._used_templates = None
        
        for key, value in kwargs.iteritems():
            setattr(self, key, value)
            
        self.logger = logging.getLogger('optimus')
    
    def __repr__(self):
        return "<Page dest:{destination}>".format(destination=self.destination)
    
    def get_context(self):
        self.context.update({
            'page_title': self.get_title(),
            'page_destination': self.get_destination(),
            'page_relative_position': self.get_relative_position(),
            'page_lang': self.get_lang(),
            'page_template_name': self.get_template_name(),
        })
        self.logger.debug(" - Initial context: %s", self.context)
        return self.context
    
    def get_relative_position(self):
        """
        Return the relative path position from the destination file to the root
        
        This is either something like "../../" if the destination is in subdirectories 
        or "./" if at the root
        """
        return ((len(self.get_destination().split("/"))-1)*"../" or "./")
    
    def get_title(self):
        return self.title
    
    def get_lang(self):
        # Defaut lang if not defined
        if getattr(self, "lang", None) is None:
            self.lang = LangBase(code=settings.LANGUAGE_CODE)
        # If the lang attribute contains a string, assume this is the language code
        elif isinstance(getattr(self, "lang"), basestring):
            self.lang = LangBase(code=getattr(self, "lang"))
        return self.lang
    
    def get_template_name(self):
        return self.template_name.format(language_code=self.get_lang().code)
    
    def get_destination(self):
        return os.path.normpath(self.destination.format(language_code=self.get_lang().code))
    
    def render(self, env):
        """
        Take the Jinja2 environment as required argument. Return 
        the HTML compiled from the template with his context.
        """
        self.env = env
        context = self.get_context()
        
        template = self.env.get_template(self.get_template_name())
        
        return template.render(lang=self.get_lang(), **context)
    
    def introspect(self, env, force=False):
        """
        Take the Jinja2 environment as required argument to find all templates dependancies.
        
        Should return a list of all template dependancies.
        """
        if self._used_templates is None:
            self.env = env
            
            self._used_templates = [self.get_template_name()] + self._recursing_template_search(self.get_template_name())
            self.logger.debug(" - Used templates: %s", self._used_templates)
        return self._used_templates
    
    def _recursing_template_search(self, template_name):
        template_source = self.env.loader.get_source(self.env, template_name)[0]
        parsed_content = self.env.parse(template_source)
        
        deps = []
        for item in Jinja2Meta.find_referenced_templates(parsed_content):
            deps.append(item)
            deps += self._recursing_template_search(item)
        
        return deps

class RstPageView(PageViewBase):
    """
    View to build a page from a ReStructuredText file
    
    You need to set the ``source_filepath`` class attributes in addition to the required 
    ones from ``PageItemBase``. ``parser_settings`` is an optionnal class attribute as a 
    dict that is passed to the docutils parser. If not given, it will be filled from the 
    ``RST_PARSER_SETTINGS`` settings option if not empty.
    
    Two additionals variables will be added to the context : 
    
    * page_doc_html: the HTML produced by the parser from the rst document
    * page_doc_source: the unparsed source from the rst document
    """
    source_filepath = None
    parser_settings = {}
    
    def get_source_filepath(self):
        return self.source_filepath
    
    def get_context(self):
        context = super(RstPageView, self).get_context()
        
        rst_parser_settings = copy.deepcopy(getattr(settings, 'RST_PARSER_SETTINGS', {}))
        rst_parser_settings.update(self.parser_settings)
        
        f = open(self.get_source_filepath(), 'r')
        doc_source = f.read()
        f.close()
        parts = docutils.core.publish_parts(source=doc_source, writer=html5writer.SemanticHTML5Writer(), settings_overrides=rst_parser_settings)
        
        context.update({
            'page_doc_html': parts['fragment'],
            'page_doc_source': doc_source,
        })
        return context

class PageRegistry(object):
    """
    Index all knowed template and memorize the pages that use them
    """
    def __init__(self, elements={}):
        self.elements = {}
        self.map_dest_to_page = {}
        self.logger = logging.getLogger('optimus')
    
    def add_page(self, page, items):
        self.map_dest_to_page[page.get_destination()] = page
        
        for k in items:
            if k in self.elements:
                self.elements[k].add(page.get_destination())
            else:
                self.elements[k] = set([page.get_destination()])
    
    def get_pages_from_dependency(self, template_name):
        """
        Return the pages object list that are dependent of the given template name
        
        This method is not safe out of the context of scanned pages, because it use 
        an internal map builded from the scan use by the add_page method. In short, it 
        will raise a KeyError exception for every destination that is doesn't known from 
        the internal map.
        """
        if template_name not in self.elements:
            self.logger.warning("Given template name is not in the page registry: %s", template_name)
            return []
        dependancies = self.elements[template_name]
        return [self.map_dest_to_page[item] for item in dependancies]

class PageBuilder(object):
    """
    Builder class to init Jinja2 environment and build the given pages
    """
    def __init__(self, jinja_env=None, assets_env=None, dry_run=False):
        self.logger = logging.getLogger('optimus')
        
        self.assets_env = assets_env
        
        self.internationalized = False
        self.translations = {}
        
        self.jinja_env = jinja_env or self.get_environnement(assets_env)
        self.set_globals()
        self.logger.debug("PageBuilder initialized")
        
        self.registry = PageRegistry()
        self.dry_run = dry_run # Not really used yet
    
    def get_environnement(self, assets_env=None):
        """
        Init the Jinja environment
        """
        exts = []
        self.logger.debug("No Jinja2 environment given, initializing a new environment")
        
        # It the assets environment is given, active the Jinja extension to use webassets
        if self.assets_env is not None:
            exts.append(AssetsExtension)
        
        # Enabled Jinja extensions
        for ext in settings.JINJA_EXTENSIONS:
            exts.append(ext)
        
        # Active i18n behaviors if i18n extension is loaded and Babel has been imported
        if 'jinja2.ext.i18n' in exts and support is not None:
            self.internationalized = True
            self.logger.debug("'i18n' enabled")
        
        # Boot Jinja environment
        env = Jinja2Environment(loader=FileSystemLoader(settings.TEMPLATES_DIR), extensions=exts)
        
        if assets_env:
            env.assets_environment = assets_env
            
        return env
    
    def set_globals(self):
        """
        Init the Jinja environment
        
        Register the webassets environment if any
        """
        self.jinja_env.globals.update({
            'debug': settings.DEBUG,
            'SITE': {
                'name': settings.SITE_NAME,
                'domain': settings.SITE_DOMAIN,
                'web_url': 'http://%s' % settings.SITE_DOMAIN,
            },
            'STATIC_URL': settings.STATIC_URL,
        })
    
    def scan_bulk(self, page_list):
        """
        Scan all the given pages to set them their dependancies
        
        Return all used templates from pages and their template dependancies
        """
        self.logger.info("Starting page builds")
        
        if not page_list:
            self.logger.warning("Page scanning skipped as there are no registered pages")
            return None
        
        knowed = set([])
        for page in page_list:
            finded = self.scan_item(page)
            self.registry.add_page(page, finded)
            knowed.update(finded)
            
        #import pprint
        #pp = pprint.PrettyPrinter(indent=4)
        #pp.pprint(self.registry.elements)
        #print "="*60
        #pp.pprint(self.registry.map_dest_to_page)
        #print
        return knowed
    
    def scan_item(self, page_item):
        """
        Scan the given page
        
        Return a list of all used templates by the page
        """
        self.logger.info(' Scanning page: %s', page_item.get_destination())
        
        return page_item.introspect(self.jinja_env)
    
    def build_bulk(self, page_list):
        """
        Build all given pages
        
        Return all the effective builded pages
        """
        self.logger.info("Starting page builds")
        
        if not page_list:
            self.logger.warning("Page management skipped as there are no registered pages")
            return None
        
        builded = []
        for page in page_list:
            builded.append( self.build_item(page) )
            
        return builded
    
    def get_translation_for_item(self, page_item):
        """
        Try to load the translations for the page language if any, then install it in Jinja2
        
        It does not reload a language translations if a previous page has allready loaded it
        """
        # Get the page language object if any, else the default one
        lang = page_item.get_lang()
        # Load language translations only if it have not been yet
        if lang.code not in self.translations:
            self.logger.debug(' - Loading translations for locale: %s - %s', lang.code, lang)
            self.translations[lang.code] = support.Translations.load(settings.LOCALES_DIR, lang.code, 'messages')
        
        # Install it in the Jinja env
        self.jinja_env.install_gettext_translations(self.translations[lang.code], newstyle=False)
    
    def build_item(self, page_item):
        """
        Build the given page
        
        Return the destination path of the builded page
        """
        self.logger.info(' Building page: %s', page_item.get_destination())
        
        # Optional i18n
        if self.internationalized:
            self.get_translation_for_item(page_item)
        
        # Template render
        content = page_item.render(self.jinja_env)
        
        destination_path = os.path.join(settings.PUBLISH_DIR, page_item.get_destination())
        # Creating destination path if needed
        destination_dir, destination_file = os.path.split(destination_path)
        if not os.path.exists(destination_dir):
            self.logger.debug(' - Creating new directory : %s', destination_dir)
            if not self.dry_run:
                os.makedirs(destination_dir)
        # Write it
        self.logger.debug(' - Writing to: %s', destination_path)
        if not self.dry_run:
            fp = open(destination_path, 'w')
            fp.write(content.encode('utf-8'))
            fp.close()
        
        return destination_path



# -*- coding: utf-8 -*-
import copy, logging

from webassets import Environment as AssetsEnvironment

from optimus.conf import settings

class AssetRegistry(object):
    """
    Index all knowed files from registered bundles
    """
    def __init__(self, elements={}):
        self.elements = {}
        self.map_dest_to_bundle = {}
        self.logger = logging.getLogger('optimus')
    
    def add_bundle(self, bundle): #, items):
        name = bundle._internal_env_name
        
        for item in bundle.contents:
            self.map_dest_to_bundle[item] = name
    
    def get_bundle_from_dependency(self, asset_name):
        """
        Return the bundles object list that are dependent of the given template name
        
        This method is not safe out of the context of scanned bundles, because it use 
        an internal map builded from the scan use by the add_bundle method. In short, it 
        will raise a KeyError exception for every destination that is doesn't known from 
        the internal map.
        """
        if asset_name not in self.elements:
            self.logger.warning("Given asset name is not in the bundle registry: %s", asset_name)
            return []
        dependancies = self.elements[asset_name]
        return [self.map_dest_to_bundle[item] for item in dependancies]

def register_assets():
    """
    Initialize webassets environment and its bundles
    
    NOTE: The asset bundles building is lazy, webassets only do building when he is 
          invoked by his template tag **assets** and if it detect that a file in a 
          bundle has changed.
    """
    logger = logging.getLogger('optimus')
    if not settings.ENABLED_BUNDLES:
        logger.warning("Asset registering skipped as there are no enabled bundles")
        return None
    logger.info("Starting asset registering")
    
    # Assets bundles
    AVAILABLE_BUNDLES = getattr(settings, 'BUNDLES', {})
    
    # Initialize webassets environment
    assets_env = AssetsEnvironment()
    assets_env.debug = settings.DEBUG
    assets_env.url = settings.STATIC_URL
    assets_env.directory = settings.STATIC_DIR
    assets_env.load_path = [settings.SOURCES_DIR]
    assets_env.cache = settings.WEBASSETS_CACHE
    
    #
    assets_env.optimus_registry = AssetRegistry()

    # Register enabled assets bundles
    for bundle_name in settings.ENABLED_BUNDLES:
        logger.debug("Registering bundle: %s", bundle_name)
        # Little trick because Bundle does not know their used name in the webassets 
        # environment
        AVAILABLE_BUNDLES[bundle_name]._internal_env_name = bundle_name
        
        assets_env.register(bundle_name, AVAILABLE_BUNDLES[bundle_name])
        assets_env.optimus_registry.add_bundle(AVAILABLE_BUNDLES[bundle_name])
        
    # for debugging purpopse
    for bundle_name in settings.ENABLED_BUNDLES:
        logger.info(" Processing: %s", assets_env[bundle_name].resolve_output())
        # TODO: conditionnal on the log level to avoid to loop on multiple items if not 
        #       in a debug log level
        for url in assets_env[bundle_name].urls():
            logger.debug(" - %s", url)
    
    return assets_env



# -*- coding: utf-8 -*-
"""
The main method to install the console script
"""
from argh import ArghParser

from optimus import __version__
from optimus.cli.build import build
from optimus.cli.watch import watch
from optimus.cli.init import init
from optimus.cli.po import po

def main():
    parser = ArghParser()
    parser.add_argument('-v', '--version', action='version', version=__version__)
    enabled_commands = [init, build, watch, po]
    
    # Enabling runserver command if cherrypy is installed
    try:
        import cherrypy
    except ImportError:
        pass
    else:
        from optimus.cli.runserver import runserver
        enabled_commands.append(runserver)
    
    parser.add_commands(enabled_commands)
    parser.dispatch()
